/*
 * This file is part of the Orpheus project.
 *
 * Copyright (C) LTR s.r.o. - All Rights Reserved
 *
 * Proprietary and confidential. You will be shot when used in other project.
 *
 * Written by Frantisek Burian <BuFran@seznam.cz>
 */
#include <sys/formaters.h>

using namespace sys::streams;

// f[sign][pad][padding][dot][decimals]

bool sys::formaters::f(ostm *ostm, const char *fmt, va_list *val)
{
  double flt = va_arg(*val, double);

  bool print_plus = false;
  int padding = 0;
  int decimals = 0;
  bool parse_decimals = false;

  for (int pos=1; fmt[pos] != '\0'; pos++) {
    switch (fmt[pos]) {
    case '+':
    case '-':
      if (pos != 0)   // bad format, allowed only on beginning
        return false;

      print_plus = true;
      continue;

    case '.':
      if (parse_decimals)
        return false;

      parse_decimals = true;
      continue;

    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
      if (!parse_decimals)
        padding = padding * 10 + (fmt[pos] - '0');
      else
        decimals = decimals * 10 + (fmt[pos] - '0');
      continue;

    default:
      return false; // bad format !
    }
  }

  for (int i=0;i<decimals;i++)
    flt *= 10.0f;

  int v = (int)(flt);

  char temp[16];
  int p = 0;
  bool print_minus = v < 0;

  if (v < 0)
    v = -v;

  while (v != 0) {
    temp[p++] = '0' + (v % 10);
    v /= 10;
  }

  while (p <= decimals)
    temp[p++] = '0';

  // todo Pad to specified length

  // todo inhibit plus

  if (print_plus || print_minus) {
    if (!ostm->put(ostm, print_minus ? '-' : '+'))
      return false;
  }

  while (p > 0) {
    --p;
    if (!ostm->put(ostm, temp[p]))
      return false;

    if (p == decimals) {
      if (!ostm->put(ostm, '.'))
        return false;
    }
  }

  return true;
}
